from django.contrib.contenttypes.fields import GenericForeignKey
from django.db import models
from polymorphic.models import PolymorphicModel

from time_wizard.models.abstract import (
    AbstractAbsoluteRangePeriod, AbstractHolidayRangePeriod,
)
from time_wizard.workarounds import NON_POLYMORPHIC_CASCADE


class PeriodModel(PolymorphicModel):
    """
    Parent model for custom periods.
    """
    content_type = models.ForeignKey(
        'contenttypes.ContentType',
        on_delete=NON_POLYMORPHIC_CASCADE,
        related_name='periods',
    )
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey()

    def contains(self, date_time):
        raise NotImplementedError("Override method in child class!")


class AbsolutePeriodModel(AbstractAbsoluteRangePeriod, PeriodModel):
    """
    Period child model for absolute time ranges from start to end.
    """
    class Meta:
        # TODO: only overriden to avoid a migration, remove when new migration
        # is created anyways
        base_manager_name = 'objects'
        abstract = False


class HolidayRangePeriodModel(AbstractHolidayRangePeriod, PeriodModel):
    """
    Period child model for holiday time ranges. Select a holiday and a range
    before, after or both from this date.
    """
    class Meta:
        # TODO: only overriden to avoid a migration, remove when new migration
        # is created anyways
        base_manager_name = 'objects'
        abstract = False



import datetime

import holidays
from django.db import models
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from time_wizard.conf import TIME_WIZARD_COUNTRIES


class AbstractAbsoluteRangePeriod(models.Model):
    """
    Abstract base class for absolute time ranges.
    """
    start = models.DateTimeField(
        blank=True,
        null=True,
    )
    end = models.DateTimeField(
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True

    def contains(self, date_time):
        if self.start and self.start > date_time:
            return False
        if self.end and self.end <= date_time:
            return False
        return True

    def __str__(self):
        return 'Absolute Range [%s - %s]' % (self.start, self.end)


class AbstractHolidayRangePeriod(models.Model):
    """
    Abstract base class for holiday time ranges.
    """
    time_value = models.PositiveSmallIntegerField(
        default=0,
        help_text=_('Amount of time between this and the selected holiday.'),
    )
    TIME_UNIT_CHOICES = (
        ('1', _('Minute')),
        ('60', _('Hour')),
        ('1440', _('Day')),
        ('10080', _('Week')),
    )
    time_unit = models.CharField(
        blank=True,
        choices=TIME_UNIT_CHOICES,
        max_length=16,
        help_text=_('Unit for the chosen time_value.'),
    )
    RANGE_POSITION_CHOICES = (
        ('before', _('Before')),
        ('after', _('After')),
        ('both', _('Both')),
    )
    range_position = models.CharField(
        blank=True,
        choices=RANGE_POSITION_CHOICES,
        max_length=8,
        help_text=_('Choose if a given time range is either before, after or '
                    'before and after the selected holiday.'),
    )
    start_time = models.TimeField(
        blank=True,
        null=True,
    )
    end_time = models.TimeField(
        blank=True,
        null=True,
    )
    country = models.CharField(
        choices=((c, c) for c in TIME_WIZARD_COUNTRIES),
        max_length=8,
    )
    province = models.CharField(
        blank=True,
        max_length=8,
    )
    holiday = models.CharField(
        max_length=64,
    )

    class Meta:
        abstract = True

    def contains(self, date_time):
        holiday_cls = getattr(holidays, self.country, False)
        if holiday_cls:
            kwargs = {'years': now().year}
            if self.province:
                kwargs.update({'prov': self.province})
            holiday_dict = holiday_cls(**kwargs)
            selected_date = self._get_selected_holiday_date(holiday_dict)
            if selected_date is not None:
                date_time = date_time.replace(tzinfo=None)
                return self._contains_selected_range(date_time, selected_date)
        return False

    def _get_selected_holiday_date(self, holiday_dict):
        selected_date = None
        for d, n in holiday_dict.items():
            if n == self.holiday:
                selected_date = d
                break
        return selected_date

    def _contains_selected_range(self, date_time, selected_date):
        time_zero = datetime.time(0)
        delta1 = datetime.timedelta(minutes=0)
        delta2 = datetime.timedelta(minutes=0)
        mins = self.time_value * int(self.time_unit) if self.time_unit else 0
        if self.range_position == 'before':
            delta1 = datetime.timedelta(minutes=mins * -1)
        elif self.range_position == 'after':
            delta2 = datetime.timedelta(minutes=mins)
        elif self.range_position == 'both':
            delta1 = datetime.timedelta(minutes=mins * -1)
            delta2 = datetime.timedelta(minutes=mins)
        date1 = datetime.datetime.combine(
            selected_date + delta1,
            self.start_time if self.start_time else time_zero)
        d3 = datetime.timedelta(days=1 if not self.end_time else 0)
        date2 = datetime.datetime.combine(
            selected_date + delta2 + d3,
            self.end_time if self.end_time else time_zero)
        if date_time < date1 or date_time > date2:
            return False
        return True

    def __str__(self):
        return 'HolidayRange [%s - %s]' % (self.country, self.holiday)



# flake8: noqa
# Generated by Django 2.1.9 on 2019-07-06 08:33

import django.db.models.deletion
import time_wizard.workarounds
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('time_wizard', '0002_delete_models'),
    ]

    operations = [
        migrations.CreateModel(
            name='PeriodModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
            ],
            options={
                'base_manager_name': 'objects',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TimeWizardModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, help_text='Optional name to identify the given TimeWizard.', max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='AbsolutePeriodModel',
            fields=[
                ('periodmodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='time_wizard.PeriodModel')),
                ('start', models.DateTimeField(blank=True, null=True)),
                ('end', models.DateTimeField(blank=True, null=True)),
            ],
            options={
                'base_manager_name': 'objects',
                'abstract': False,
            },
            bases=('time_wizard.periodmodel',),
        ),
        migrations.CreateModel(
            name='HolidayRangePeriodModel',
            fields=[
                ('periodmodel_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='time_wizard.PeriodModel')),
                ('time_value', models.PositiveSmallIntegerField(default=0, help_text='Amount of time between this and the selected holiday.')),
                ('time_unit', models.CharField(blank=True, choices=[('1', 'Minute'), ('60', 'Hour'), ('1440', 'Day'), ('10080', 'Week')], help_text='Unit for the chosen time_value.', max_length=16)),
                ('range_position', models.CharField(blank=True, choices=[('before', 'Before'), ('after', 'After'), ('both', 'Both')], help_text='Choose if a given time range is either before, after or before and after the selected holiday.', max_length=8)),
                ('start_time', models.TimeField(blank=True, null=True)),
                ('end_time', models.TimeField(blank=True, null=True)),
                ('country', models.CharField(choices=[('AR', 'AR'), ('AT', 'AT'), ('AU', 'AU'), ('BE', 'BE'), ('BG', 'BG'), ('BR', 'BR'), ('BY', 'BY'), ('CA', 'CA'), ('CH', 'CH'), ('CO', 'CO'), ('CZ', 'CZ'), ('DE', 'DE'), ('DK', 'DK'), ('ECB', 'ECB'), ('ES', 'ES'), ('FI', 'FI'), ('FRA', 'FRA'), ('HND', 'HND'), ('HR', 'HR'), ('HU', 'HU'), ('IE', 'IE'), ('IND', 'IND'), ('IT', 'IT'), ('JP', 'JP'), ('LT', 'LT'), ('LU', 'LU'), ('MX', 'MX'), ('NL', 'NL'), ('NO', 'NO'), ('NZ', 'NZ'), ('PL', 'PL'), ('PT', 'PT'), ('PTE', 'PTE'), ('RU', 'RU'), ('SE', 'SE'), ('SI', 'SI'), ('SK', 'SK'), ('TAR', 'TAR'), ('UA', 'UA'), ('UK', 'UK'), ('US', 'US'), ('ZA', 'ZA')], max_length=8)),
                ('province', models.CharField(blank=True, max_length=8)),
                ('holiday', models.CharField(max_length=64)),
            ],
            options={
                'base_manager_name': 'objects',
                'abstract': False,
            },
            bases=('time_wizard.periodmodel',),
        ),
        migrations.AddField(
            model_name='periodmodel',
            name='content_type',
            field=models.ForeignKey(on_delete=time_wizard.workarounds.NON_POLYMORPHIC_CASCADE, related_name='periods', to='contenttypes.ContentType'),
        ),
        migrations.AddField(
            model_name='periodmodel',
            name='polymorphic_ctype',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_time_wizard.periodmodel_set+', to='contenttypes.ContentType'),
        ),
    ]



import holidays
from django.shortcuts import render
from django.utils.timezone import now

from time_wizard.conf import TIME_WIZARD_COUNTRY_PROVINCES


def load_holidays(request):
    country = request.GET.get('country', '')
    province = request.GET.get('province', '')
    holiday_cls = getattr(holidays, country, False)
    context = {}
    if holiday_cls is not False:
        kwargs = {'years': now().year}
        if province:
            kwargs.update({'prov': province})
        context.update({'holidays': holiday_cls(**kwargs)})
    t = 'time_wizard/time_wizard_holiday_range_list_options.html'
    return render(request, t, context)


def load_provinces(request):
    country = request.GET.get('country', '')
    context = {}
    if country in TIME_WIZARD_COUNTRY_PROVINCES:
        context.update({'provinces': TIME_WIZARD_COUNTRY_PROVINCES[country]})
    t = 'time_wizard/time_wizard_province_range_list_options.html'
    return render(request, t, context)



from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _

from time_wizard.mixins import TimeWizardInlineMixin


class TimeWizardModel(TimeWizardInlineMixin, models.Model):
    """
    Model for the TimeWizard.
    """
    name = models.CharField(
        blank=True,
        help_text=_('Optional name to identify the given TimeWizard.'),
        max_length=255,
    )

    def __str__(self):
        if self.name:
            return self.name
        else:
            ct = ContentType.objects.get_for_model(self)
            return ' - '.join(
                [str(s) for s in self.periods.model.objects.filter(
                 content_type=ct, object_id=self.id)])



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-10-10 12:57
from __future__ import unicode_literals

import django.db.models.deletion
import time_wizard.workarounds
from django.db import migrations, models
from time_wizard.conf import TIME_WIZARD_COUNTRIES


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='PeriodModel',
            fields=[
                ('id', models.AutoField(
                    auto_created=True,
                    primary_key=True,
                    serialize=False,
                    verbose_name='ID')),
            ],
            options={
                'manager_inheritance_from_future': True,
            },
        ),
        migrations.CreateModel(
            name='TimeWizardModel',
            fields=[
                ('id', models.AutoField(
                    auto_created=True,
                    primary_key=True,
                    serialize=False,
                    verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='AbsolutePeriodModel',
            fields=[
                ('periodmodel_ptr', models.OneToOneField(
                    auto_created=True,
                    on_delete=django.db.models.deletion.CASCADE,
                    parent_link=True,
                    primary_key=True,
                    serialize=False,
                    to='time_wizard.PeriodModel')),
                ('start', models.DateTimeField(
                    blank=True,
                    null=True)),
                ('end', models.DateTimeField(
                    blank=True,
                    null=True)),
            ],
            options={
                'manager_inheritance_from_future': True,
            },
            bases=('time_wizard.periodmodel',),
        ),
        migrations.CreateModel(
            name='HolidayRangePeriodModel',
            fields=[
                ('periodmodel_ptr', models.OneToOneField(
                    auto_created=True,
                    on_delete=django.db.models.deletion.CASCADE,
                    parent_link=True,
                    primary_key=True,
                    serialize=False,
                    to='time_wizard.PeriodModel')),
                ('time_value', models.PositiveSmallIntegerField(
                    default=0,
                    help_text='Amount of time between this and the selected '
                              'holiday.')),
                ('time_unit', models.CharField(
                    blank=True,
                    choices=[
                        ('1', 'Minute'),
                        ('60', 'Hour'),
                        ('1440', 'Day'),
                        ('10080', 'Week')],
                    help_text='Unit for the chosen time_value.',
                    max_length=16)),
                ('range_position', models.CharField(
                    blank=True,
                    choices=[
                        ('before', 'Before'),
                        ('after', 'After'),
                        ('both', 'Both')],
                    help_text='Choose if a given time range is either before, '
                              'after or before and after the selected '
                              'holiday.',
                    max_length=8)),
                ('start_time', models.TimeField(
                    blank=True,
                    null=True)),
                ('end_time', models.TimeField(
                    blank=True,
                    null=True)),
                ('country', models.CharField(
                    choices=[(c, c) for c in TIME_WIZARD_COUNTRIES],
                    max_length=8)),
                ('province', models.CharField(
                    blank=True,
                    max_length=8)),
                ('holiday', models.CharField(
                    max_length=64)),
            ],
            options={
                'manager_inheritance_from_future': True,
            },
            bases=('time_wizard.periodmodel',),
        ),
        migrations.AddField(
            model_name='periodmodel',
            name='container',
            field=models.ForeignKey(
                    on_delete=time_wizard.workarounds.NON_POLYMORPHIC_CASCADE,
                    related_name='periods',
                    to='time_wizard.TimeWizardModel'),
        ),
        migrations.AddField(
            model_name='periodmodel',
            name='polymorphic_ctype',
            field=models.ForeignKey(
                    editable=False,
                    null=True,
                    on_delete=django.db.models.deletion.CASCADE,
                    related_name='polymorphic_time_wizard.periodmodel_set+',
                    to='contenttypes.ContentType'),
        ),
    ]



from setuptools import find_packages, setup
from time_wizard import __version__


setup(
    name='django-time-wizard',
    version=__version__,
    packages=find_packages(exclude=['tests']),
    install_requires=[
        'django>=1.11.27,<3.0',
        'django-polymorphic>=0.7',
        'holidays==0.9.10',
    ],
    include_package_data=True,
    description='Date and time dependend content manipulation',
    long_description=open('README.rst').read(),
    url='https://github.com/wfehr/django-time-wizard',
    download_url='https://github.com/wfehr/django-time-wizard/tarball/master',
    author='Wolfgang Fehr',
    author_email='dev@wfehr.de',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    test_suite='tests.settings.run',
)
